function noop() { }
function run(fn) {
    return fn();
}
function blank_object() {
    return Object.create(null);
}
function run_all(fns) {
    fns.forEach(run);
}
function is_function(thing) {
    return typeof thing === 'function';
}
function safe_not_equal(a, b) {
    return a != a ? b == b : a !== b || ((a && typeof a === 'object') || typeof a === 'function');
}

function append(target, node) {
    target.appendChild(node);
}
function insert(target, node, anchor) {
    target.insertBefore(node, anchor || null);
}
function detach(node) {
    node.parentNode.removeChild(node);
}
function element(name) {
    return document.createElement(name);
}
function listen(node, event, handler, options) {
    node.addEventListener(event, handler, options);
    return () => node.removeEventListener(event, handler, options);
}
function attr(node, attribute, value) {
    if (value == null)
        node.removeAttribute(attribute);
    else if (node.getAttribute(attribute) !== value)
        node.setAttribute(attribute, value);
}
function children(element) {
    return Array.from(element.childNodes);
}
function custom_event(type, detail) {
    const e = document.createEvent('CustomEvent');
    e.initCustomEvent(type, false, false, detail);
    return e;
}

let current_component;
function set_current_component(component) {
    current_component = component;
}
function get_current_component() {
    if (!current_component)
        throw new Error(`Function called outside component initialization`);
    return current_component;
}
function onMount(fn) {
    get_current_component().$$.on_mount.push(fn);
}
function afterUpdate(fn) {
    get_current_component().$$.after_update.push(fn);
}
function createEventDispatcher() {
    const component = get_current_component();
    return (type, detail) => {
        const callbacks = component.$$.callbacks[type];
        if (callbacks) {
            // TODO are there situations where events could be dispatched
            // in a server (non-DOM) environment?
            const event = custom_event(type, detail);
            callbacks.slice().forEach(fn => {
                fn.call(component, event);
            });
        }
    };
}

const dirty_components = [];
const binding_callbacks = [];
const render_callbacks = [];
const flush_callbacks = [];
const resolved_promise = Promise.resolve();
let update_scheduled = false;
function schedule_update() {
    if (!update_scheduled) {
        update_scheduled = true;
        resolved_promise.then(flush);
    }
}
function add_render_callback(fn) {
    render_callbacks.push(fn);
}
let flushing = false;
const seen_callbacks = new Set();
function flush() {
    if (flushing)
        return;
    flushing = true;
    do {
        // first, call beforeUpdate functions
        // and update components
        for (let i = 0; i < dirty_components.length; i += 1) {
            const component = dirty_components[i];
            set_current_component(component);
            update(component.$$);
        }
        dirty_components.length = 0;
        while (binding_callbacks.length)
            binding_callbacks.pop()();
        // then, once components are updated, call
        // afterUpdate functions. This may cause
        // subsequent updates...
        for (let i = 0; i < render_callbacks.length; i += 1) {
            const callback = render_callbacks[i];
            if (!seen_callbacks.has(callback)) {
                // ...so guard against infinite loops
                seen_callbacks.add(callback);
                callback();
            }
        }
        render_callbacks.length = 0;
    } while (dirty_components.length);
    while (flush_callbacks.length) {
        flush_callbacks.pop()();
    }
    update_scheduled = false;
    flushing = false;
    seen_callbacks.clear();
}
function update($$) {
    if ($$.fragment !== null) {
        $$.update();
        run_all($$.before_update);
        const dirty = $$.dirty;
        $$.dirty = [-1];
        $$.fragment && $$.fragment.p($$.ctx, dirty);
        $$.after_update.forEach(add_render_callback);
    }
}
const outroing = new Set();
function transition_in(block, local) {
    if (block && block.i) {
        outroing.delete(block);
        block.i(local);
    }
}
function mount_component(component, target, anchor) {
    const { fragment, on_mount, on_destroy, after_update } = component.$$;
    fragment && fragment.m(target, anchor);
    // onMount happens before the initial afterUpdate
    add_render_callback(() => {
        const new_on_destroy = on_mount.map(run).filter(is_function);
        if (on_destroy) {
            on_destroy.push(...new_on_destroy);
        }
        else {
            // Edge case - component was destroyed immediately,
            // most likely as a result of a binding initialising
            run_all(new_on_destroy);
        }
        component.$$.on_mount = [];
    });
    after_update.forEach(add_render_callback);
}
function destroy_component(component, detaching) {
    const $$ = component.$$;
    if ($$.fragment !== null) {
        run_all($$.on_destroy);
        $$.fragment && $$.fragment.d(detaching);
        // TODO null out other refs, including component.$$ (but need to
        // preserve final state?)
        $$.on_destroy = $$.fragment = null;
        $$.ctx = [];
    }
}
function make_dirty(component, i) {
    if (component.$$.dirty[0] === -1) {
        dirty_components.push(component);
        schedule_update();
        component.$$.dirty.fill(0);
    }
    component.$$.dirty[(i / 31) | 0] |= (1 << (i % 31));
}
function init(component, options, instance, create_fragment, not_equal, props, dirty = [-1]) {
    const parent_component = current_component;
    set_current_component(component);
    const prop_values = options.props || {};
    const $$ = component.$$ = {
        fragment: null,
        ctx: null,
        // state
        props,
        update: noop,
        not_equal,
        bound: blank_object(),
        // lifecycle
        on_mount: [],
        on_destroy: [],
        before_update: [],
        after_update: [],
        context: new Map(parent_component ? parent_component.$$.context : []),
        // everything else
        callbacks: blank_object(),
        dirty
    };
    let ready = false;
    $$.ctx = instance
        ? instance(component, prop_values, (i, ret, ...rest) => {
            const value = rest.length ? rest[0] : ret;
            if ($$.ctx && not_equal($$.ctx[i], $$.ctx[i] = value)) {
                if ($$.bound[i])
                    $$.bound[i](value);
                if (ready)
                    make_dirty(component, i);
            }
            return ret;
        })
        : [];
    $$.update();
    ready = true;
    run_all($$.before_update);
    // `false` as a special case of no DOM component
    $$.fragment = create_fragment ? create_fragment($$.ctx) : false;
    if (options.target) {
        if (options.hydrate) {
            const nodes = children(options.target);
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            $$.fragment && $$.fragment.l(nodes);
            nodes.forEach(detach);
        }
        else {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            $$.fragment && $$.fragment.c();
        }
        if (options.intro)
            transition_in(component.$$.fragment);
        mount_component(component, options.target, options.anchor);
        flush();
    }
    set_current_component(parent_component);
}
class SvelteComponent {
    $destroy() {
        destroy_component(this, 1);
        this.$destroy = noop;
    }
    $on(type, callback) {
        const callbacks = (this.$$.callbacks[type] || (this.$$.callbacks[type] = []));
        callbacks.push(callback);
        return () => {
            const index = callbacks.indexOf(callback);
            if (index !== -1)
                callbacks.splice(index, 1);
        };
    }
    $set() {
        // overridden by instance, if it has props
    }
}

/**
 * Detect Element Resize
 *
 * https://github.com/sdecima/javascript-detect-element-resize
 * Sebastian Decima
 *
 * version: 0.5.3
 **/
var stylesCreated = false;

var requestFrame = (function() {
  var raf =
    window.requestAnimationFrame ||
    window.mozRequestAnimationFrame ||
    window.webkitRequestAnimationFrame ||
    function(fn) {
      return window.setTimeout(fn, 20);
    };
  return function(fn) {
    return raf(fn);
  };
})();

var cancelFrame = (function() {
  var cancel =
    window.cancelAnimationFrame ||
    window.mozCancelAnimationFrame ||
    window.webkitCancelAnimationFrame ||
    window.clearTimeout;
  return function(id) {
    return cancel(id);
  };
})();

var resetTriggers = function resetTriggers(element) {
  var triggers = element.__resizeTriggers__,
    expand = triggers.firstElementChild,
    contract = triggers.lastElementChild,
    expandChild = expand.firstElementChild;
  contract.scrollLeft = contract.scrollWidth;
  contract.scrollTop = contract.scrollHeight;
  expandChild.style.width = expand.offsetWidth + 1 + 'px';
  expandChild.style.height = expand.offsetHeight + 1 + 'px';
  expand.scrollLeft = expand.scrollWidth;
  expand.scrollTop = expand.scrollHeight;
};

function checkTriggers(element) {
  return (
    element.offsetWidth != element.__resizeLast__.width || element.offsetHeight != element.__resizeLast__.height
  );
}

function scrollListener(e) {
  var element = this;
  resetTriggers(this);
  if (this.__resizeRAF__) cancelFrame(this.__resizeRAF__);
  this.__resizeRAF__ = requestFrame(function() {
    if (checkTriggers(element)) {
      element.__resizeLast__.width = element.offsetWidth;
      element.__resizeLast__.height = element.offsetHeight;
      element.__resizeListeners__.forEach(function(fn) {
        fn.call(element, e);
      });
    }
  });
}

/* Detect CSS Animations support to detect element display/re-attach */
var animation = false,
  keyframeprefix = '',
  animationstartevent = 'animationstart',
  domPrefixes = 'Webkit Moz O ms'.split(' '),
  startEvents = 'webkitAnimationStart animationstart oAnimationStart MSAnimationStart'.split(' '),
  pfx = '';
{
  var elm = document.createElement('fakeelement');
  if (elm.style.animationName !== undefined) {
    animation = true;
  }

  if (animation === false) {
    for (var i = 0; i < domPrefixes.length; i++) {
      if (elm.style[domPrefixes[i] + 'AnimationName'] !== undefined) {
        pfx = domPrefixes[i];
        keyframeprefix = '-' + pfx.toLowerCase() + '-';
        animationstartevent = startEvents[i];
        animation = true;
        break;
      }
    }
  }
}

var animationName = 'resizeanim';
var animationKeyframes =
  '@' + keyframeprefix + 'keyframes ' + animationName + ' { from { opacity: 0; } to { opacity: 0; } } ';
var animationStyle = keyframeprefix + 'animation: 1ms ' + animationName + '; ';

function createStyles() {
  if (!stylesCreated) {
    //opacity:0 works around a chrome bug https://code.google.com/p/chromium/issues/detail?id=286360
    var css =
      (animationKeyframes ? animationKeyframes : '') +
      '.resize-triggers { ' +
      (animationStyle ? animationStyle : '') +
      'visibility: hidden; opacity: 0; } ' +
      '.resize-triggers, .resize-triggers > div, .contract-trigger:before { content: " "; display: block; position: absolute; top: 0; left: 0; height: 100%; width: 100%; overflow: hidden; } .resize-triggers > div { background: #eee; overflow: auto; } .contract-trigger:before { width: 200%; height: 200%; }',
      head = document.head || document.getElementsByTagName('head')[0],
      style = document.createElement('style');

    style.type = 'text/css';
    if (style.styleSheet) {
      style.styleSheet.cssText = css;
    } else {
      style.appendChild(document.createTextNode(css));
    }

    head.appendChild(style);
    stylesCreated = true;
  }
}

var addResizeListener = function(element, fn) {
  if (!element) {
    throw new TypeError('Expected first argument to be provided but was ' + element);
  }
  if (!element instanceof HTMLElement) {
    throw new TypeError('Expected first argument to be a HTMLElement but was ' + element.constructor.name);
  }
  if (!fn || typeof fn !== 'function') {
    throw new TypeError('Expected second argument to be a function but got ' + fn);
  }
  if (!element.__resizeTriggers__) {
    if (getComputedStyle(element).position == 'static') element.style.position = 'relative';
    createStyles();
    element.__resizeLast__ = {};
    element.__resizeListeners__ = [];
    (element.__resizeTriggers__ = document.createElement('div')).className = 'resize-triggers';
    element.__resizeTriggers__.innerHTML =
      '<div class="expand-trigger"><div></div></div>' + '<div class="contract-trigger"></div>';
    element.appendChild(element.__resizeTriggers__);
    resetTriggers(element);
    element.addEventListener('scroll', scrollListener, true);

    /* Listen for a css animation to detect element display/re-attach */
    animationstartevent &&
      element.__resizeTriggers__.addEventListener(animationstartevent, function(e) {
        if (e.animationName == animationName) resetTriggers(element);
      });
  }
  element.__resizeListeners__.push(fn);
  return function unsubscribeResizeListener() {
    removeResizeListener(element, fn);
  }
};

var removeResizeListener = function(element, fn) {
  element.__resizeListeners__.splice(element.__resizeListeners__.indexOf(fn), 1);
  if (!element.__resizeListeners__.length) {
    element.removeEventListener('scroll', scrollListener);
    element.__resizeTriggers__ = !element.removeChild(element.__resizeTriggers__);
  }
};

var addResizeListener_1 = addResizeListener;
var removeResizeListener_1 = removeResizeListener;
addResizeListener_1.removeResizeListener = removeResizeListener_1;

/* src\Scrollbar.svelte generated by Svelte v3.20.1 */

function add_css() {
	var style = element("style");
	style.id = "svelte-18o85o7-style";
	style.textContent = ".container.svelte-18o85o7{box-sizing:border-box;height:var(--height);width:var(--width);padding:var(--padding);background-color:var(--backgroundColor)}.nub.svelte-18o85o7{width:100%;height:100%;border-radius:var(--nubBorderRadius);position:relative;top:0;background-color:var(--nubColor);transition:var(--hoverTransition)}.nub.svelte-18o85o7:hover{background-color:var(--nubHovered)}";
	append(document.head, style);
}

function create_fragment(ctx) {
	let div1;
	let div0;
	let div0_style_value;
	let div1_style_value;
	let dispose;

	return {
		c() {
			div1 = element("div");
			div0 = element("div");
			attr(div0, "class", "nub svelte-18o85o7");
			attr(div0, "style", div0_style_value = "\n\t\t\t" + `${/*horizontal*/ ctx[0] ? "width" : "height"}: ${/*viewable*/ ctx[7] / /*total*/ ctx[6] * 100}%;` + "\n\t\t\t" + `${/*horizontal*/ ctx[0] ? "left" : "top"}: ${/*position*/ ctx[5] / /*total*/ ctx[6] * 100}%;` + "\n\t\t\t" + /*nubStyle*/ ctx[3] + "\n\t\t");
			attr(div0, "draggable", "false");
			attr(div1, "class", "container svelte-18o85o7");
			attr(div1, "draggable", "false");

			attr(div1, "style", div1_style_value = "\n\t\t--nubColor: " + /*renderedColors*/ ctx[11].nub + ";\n\t\t--nubHovered: " + /*renderedColors*/ ctx[11].nubHovered + ";\n\t\t--backgroundColor: " + /*renderedColors*/ ctx[11].background + ";\n\n\t\t/* Swap width and height if horizontal */\n\t\t--width: " + (/*horizontal*/ ctx[0]
			? /*renderedStyling*/ ctx[10].height
			: /*renderedStyling*/ ctx[10].width) + ";\n\t\t--height: " + (/*horizontal*/ ctx[0]
			? /*renderedStyling*/ ctx[10].width
			: /*renderedStyling*/ ctx[10].height) + ";\n\t\t--padding: " + /*renderedStyling*/ ctx[10].padding + ";\n\n\t\tposition: " + /*renderedStyling*/ ctx[10].cssPosition + ";\n\t\ttop: " + /*renderedStyling*/ ctx[10].top + ";\n\t\tright: " + /*renderedStyling*/ ctx[10].right + ";\n\t\tleft: " + /*renderedStyling*/ ctx[10].left + ";\n\t\tbottom: " + /*renderedStyling*/ ctx[10].bottom + ";\n\n\t\t--hoverTransition: " + /*renderedStyling*/ ctx[10].hoverTransition + ";\n\t\t--nubBorderRadius: " + /*renderedStyling*/ ctx[10].borderRadius + ";\n\t\t" + /*containerStyle*/ ctx[1] + "\n\t");
		},
		m(target, anchor, remount) {
			insert(target, div1, anchor);
			append(div1, div0);
			/*div0_binding*/ ctx[26](div0);
			/*div1_binding*/ ctx[27](div1);
			if (remount) run_all(dispose);

			dispose = [
				listen(div0, "mouseover", /*mouseover_handler*/ ctx[24]),
				listen(div0, "mouseout", /*mouseout_handler*/ ctx[25]),
				listen(div1, "mouseover", /*mouseover_handler_1*/ ctx[28]),
				listen(div1, "mouseout", /*mouseout_handler_1*/ ctx[29])
			];
		},
		p(ctx, [dirty]) {
			if (dirty & /*horizontal, viewable, total, position, nubStyle*/ 233 && div0_style_value !== (div0_style_value = "\n\t\t\t" + `${/*horizontal*/ ctx[0] ? "width" : "height"}: ${/*viewable*/ ctx[7] / /*total*/ ctx[6] * 100}%;` + "\n\t\t\t" + `${/*horizontal*/ ctx[0] ? "left" : "top"}: ${/*position*/ ctx[5] / /*total*/ ctx[6] * 100}%;` + "\n\t\t\t" + /*nubStyle*/ ctx[3] + "\n\t\t")) {
				attr(div0, "style", div0_style_value);
			}

			if (dirty & /*renderedColors, horizontal, renderedStyling, containerStyle*/ 3075 && div1_style_value !== (div1_style_value = "\n\t\t--nubColor: " + /*renderedColors*/ ctx[11].nub + ";\n\t\t--nubHovered: " + /*renderedColors*/ ctx[11].nubHovered + ";\n\t\t--backgroundColor: " + /*renderedColors*/ ctx[11].background + ";\n\n\t\t/* Swap width and height if horizontal */\n\t\t--width: " + (/*horizontal*/ ctx[0]
			? /*renderedStyling*/ ctx[10].height
			: /*renderedStyling*/ ctx[10].width) + ";\n\t\t--height: " + (/*horizontal*/ ctx[0]
			? /*renderedStyling*/ ctx[10].width
			: /*renderedStyling*/ ctx[10].height) + ";\n\t\t--padding: " + /*renderedStyling*/ ctx[10].padding + ";\n\n\t\tposition: " + /*renderedStyling*/ ctx[10].cssPosition + ";\n\t\ttop: " + /*renderedStyling*/ ctx[10].top + ";\n\t\tright: " + /*renderedStyling*/ ctx[10].right + ";\n\t\tleft: " + /*renderedStyling*/ ctx[10].left + ";\n\t\tbottom: " + /*renderedStyling*/ ctx[10].bottom + ";\n\n\t\t--hoverTransition: " + /*renderedStyling*/ ctx[10].hoverTransition + ";\n\t\t--nubBorderRadius: " + /*renderedStyling*/ ctx[10].borderRadius + ";\n\t\t" + /*containerStyle*/ ctx[1] + "\n\t")) {
				attr(div1, "style", div1_style_value);
			}
		},
		i: noop,
		o: noop,
		d(detaching) {
			if (detaching) detach(div1);
			/*div0_binding*/ ctx[26](null);
			/*div1_binding*/ ctx[27](null);
			run_all(dispose);
		}
	};
}

function clamp(n, min, max) {
	if (n < min) {
		return min;
	} else if (n > max) {
		return max;
	} else {
		return n;
	}
}

//? Replaces each attribute in an obj specified by the
//? "keys" array with the value passed to the function
//? This is not a pure function
function replaceObjKeysWith(obj, keys, val) {
	for (const attr of keys) {
		obj[attr] = val;
	}
}

//? Since I couldn't find a built-in for this I made one myself
//? This goes through all the keys in the defaults object and checks
//? if they exist in the target obj, if they don't or if their value is
//? undefined, then it will create / replace that attribute's value with
//? the one in the default object
function setDefaultsByUndefined(obj, defaults) {
	const result = { ...obj };

	for (const attr of Object.keys(defaults)) {
		if (result[attr] === undefined) {
			result[attr] = defaults[attr];
		}
	}

	return result;
}

function instance($$self, $$props, $$invalidate) {
	let container;
	let nub;
	let containerBox;
	let containerContentLength; //? Content length (Bounding Client Height / Width minus padding)
	let containerLength; //? Actual length (Bounding Client Height / Width based on horizontal prop)
	const dispatch = createEventDispatcher();
	const dragging = { current: false }; //? { current } to make sure values update throughout all the callbacks
	const internalPosition = { current: 0 }; //? This is the position of the center of the scroll nub
	let { horizontal = false } = $$props;
	let { styling = {} } = $$props;

	const defaultStyling = {
		width: "18px",
		height: "100%",
		padding: "4px",
		cssPosition: "relative",
		top: "unset",
		right: "unset",
		left: "unset",
		bottom: "unset",
		hoverTransition: "unset",
		borderRadius: "10px"
	};

	let renderedStyling;
	let { colorScheme = {} } = $$props;

	const defaultColors = {
		nubClicked: "#787878",
		nubHovered: "#A8A8A8",
		nub: "#C1C1C1",
		background: "#FFF"
	};

	let renderedColors;
	let { containerStyle = "" } = $$props;
	let { containerHoveredStyle = {} } = $$props;
	let { nubStyle = "" } = $$props;
	let { nubHoveredStyle = {} } = $$props;
	let { position } = $$props;
	let prevPosition;
	let { total } = $$props;
	let { viewable } = $$props;

	function updateAll() {
		containerBox = container.getBoundingClientRect();
		$$invalidate(16, containerLength = horizontal ? containerBox.width : containerBox.height);
		const computedContainerStyle = window.getComputedStyle(container);

		const paddingFront = horizontal
		? computedContainerStyle.paddingLeft
		: computedContainerStyle.paddingTop;

		const paddingBack = horizontal
		? computedContainerStyle.paddingRight
		: computedContainerStyle.paddingBottom;

		containerContentLength = containerLength - parseFloat(paddingFront) - parseFloat(paddingBack);
	}

	onMount(() => {
		const removeResizeListener = addResizeListener_1(container, () => updateAll());
		updateAll();

		const moveNub = pos => {
			//? Nub's dimensions
			const nubBox = nub.getBoundingClientRect();

			const nubLength = horizontal ? nubBox.width : nubBox.height;
			const nubTop = horizontal ? nubBox.left : nubBox.top;

			//? Center of the nub
			const center = Math.round(nubLength / 2) + nubTop;

			//? How far to move to align the center of the nub to the cursor
			//? Because it makes most sense for the center of the nub to be
			//? aligned to your cursor instead of the top
			const delta = pos - center;

			//? internalPosition represents how far the nub moved measured
			//? (from it's top to the top of the containertop)
			internalPosition.current += delta;

			//? Limit internalPosition between 0 and the bottom of the container minus
			//? the nub's height because internalPosition represents the position
			//? at the top of the nub (otherwise the nub will only be stopped when
			//? its top touches the bottom)
			internalPosition.current = clamp(internalPosition.current, 0, containerContentLength - nubLength);

			dispatch("scroll", {
				//? Calculate the percentage scrolled but subtract the nub's height
				//? this is because we clamped the internalPosition, which is the position
				//? of the top of the nub to the total height minus the nub's height
				//? Without subtracting it here as well the percentage will end up
				//? being innacurate because it does not match up with the clamped range
				truePosition: internalPosition.current / (containerContentLength - nubLength),
				//? Here "false" position is still included because the someone using
				//? will probably want to keep a state of the scrollbar that is updated
				//? via event
				position: internalPosition.current / containerContentLength
			});
		};

		const onmousedown = e => {
			const containerBoundMin1 = horizontal ? containerBox.top : containerBox.left;
			const containerBoundMax1 = horizontal ? containerBox.bottom : containerBox.right;
			const containerBoundMin2 = horizontal ? containerBox.left : containerBox.top;
			const containerBoundMax2 = horizontal ? containerBox.right : containerBox.bottom;
			const mouseMeasure1 = horizontal ? e.y : e.x;
			const mouseMeasure2 = horizontal ? e.x : e.y;

			//? Make sure cursor is within x and y bounds of scroll container
			if (!(mouseMeasure2 < containerBoundMax2 && mouseMeasure2 > containerBoundMin2) || !(mouseMeasure1 < containerBoundMax1 && mouseMeasure1 > containerBoundMin1)) {
				return;
			}

			moveNub(horizontal ? e.x : e.y);
			dragging.current = true;
		};

		const onmouseup = () => {
			//? Reset Styles
			replaceObjKeysWith(container.style, Object.keys(containerHoveredStyle), "");

			replaceObjKeysWith(nub.style, Object.keys(nubHoveredStyle), "");
			$$invalidate(9, nub.style.backgroundColor = "", nub);
			dragging.current = false;
		};

		const onmousemove = e => {
			if (dragging.current) {
				moveNub(horizontal ? e.x : e.y);
			}
		};

		const ondrag = e => e.preventDefault();
		window.addEventListener("mousedown", onmousedown);
		window.addEventListener("mouseup", onmouseup);
		window.addEventListener("mousemove", onmousemove);

		//? To prevent dragging and totally breaking the scrollbar
		window.addEventListener("dragstart", ondrag);

		return () => {
			window.removeEventListener("mousedown", onmousedown);
			window.removeEventListener("mouseup", onmouseup);
			window.removeEventListener("mousemove", onmousemove);
			window.removeEventListener("dragstart", ondrag);
			removeResizeListener();
		};
	});

	afterUpdate(() => {
		if (dragging.current) {
			$$invalidate(9, nub.style.backgroundColor = renderedColors.nubClicked, nub);
			Object.assign(container.style, containerHoveredStyle);
			Object.assign(nub.style, nubHoveredStyle);
		}
	});

	const mouseover_handler = () => Object.assign(nub.style, nubHoveredStyle);
	const mouseout_handler = () => replaceObjKeysWith(nub.style, Object.keys(nubHoveredStyle), "");

	function div0_binding($$value) {
		binding_callbacks[$$value ? "unshift" : "push"](() => {
			$$invalidate(9, nub = $$value);
		});
	}

	function div1_binding($$value) {
		binding_callbacks[$$value ? "unshift" : "push"](() => {
			$$invalidate(8, container = $$value);
		});
	}

	const mouseover_handler_1 = () => Object.assign(container.style, containerHoveredStyle);
	const mouseout_handler_1 = () => replaceObjKeysWith(container.style, Object.keys(containerHoveredStyle), "");

	$$self.$set = $$props => {
		if ("horizontal" in $$props) $$invalidate(0, horizontal = $$props.horizontal);
		if ("styling" in $$props) $$invalidate(12, styling = $$props.styling);
		if ("colorScheme" in $$props) $$invalidate(13, colorScheme = $$props.colorScheme);
		if ("containerStyle" in $$props) $$invalidate(1, containerStyle = $$props.containerStyle);
		if ("containerHoveredStyle" in $$props) $$invalidate(2, containerHoveredStyle = $$props.containerHoveredStyle);
		if ("nubStyle" in $$props) $$invalidate(3, nubStyle = $$props.nubStyle);
		if ("nubHoveredStyle" in $$props) $$invalidate(4, nubHoveredStyle = $$props.nubHoveredStyle);
		if ("position" in $$props) $$invalidate(5, position = $$props.position);
		if ("total" in $$props) $$invalidate(6, total = $$props.total);
		if ("viewable" in $$props) $$invalidate(7, viewable = $$props.viewable);
	};

	$$self.$$.update = () => {
		if ($$self.$$.dirty & /*styling*/ 4096) {
			{
				$$invalidate(10, renderedStyling = setDefaultsByUndefined(styling, defaultStyling));
			}
		}

		if ($$self.$$.dirty & /*colorScheme*/ 8192) {
			{
				$$invalidate(11, renderedColors = setDefaultsByUndefined(colorScheme, defaultColors));
			}
		}

		if ($$self.$$.dirty & /*position, prevPosition, containerLength, total*/ 589920) {
			{
				if (position !== prevPosition && containerLength) {
					internalPosition.current = position / total * containerLength;
					$$invalidate(19, prevPosition = position);
				}
			}
		}
	};

	return [
		horizontal,
		containerStyle,
		containerHoveredStyle,
		nubStyle,
		nubHoveredStyle,
		position,
		total,
		viewable,
		container,
		nub,
		renderedStyling,
		renderedColors,
		styling,
		colorScheme,
		containerBox,
		containerContentLength,
		containerLength,
		dragging,
		internalPosition,
		prevPosition,
		dispatch,
		defaultStyling,
		defaultColors,
		updateAll,
		mouseover_handler,
		mouseout_handler,
		div0_binding,
		div1_binding,
		mouseover_handler_1,
		mouseout_handler_1
	];
}

class Scrollbar extends SvelteComponent {
	constructor(options) {
		super();
		if (!document.getElementById("svelte-18o85o7-style")) add_css();

		init(this, options, instance, create_fragment, safe_not_equal, {
			horizontal: 0,
			styling: 12,
			colorScheme: 13,
			containerStyle: 1,
			containerHoveredStyle: 2,
			nubStyle: 3,
			nubHoveredStyle: 4,
			position: 5,
			total: 6,
			viewable: 7
		});
	}
}

export default Scrollbar;
